package net.moonleay.etaorion.client

import net.fabricmc.api.ClientModInitializer
import net.fabricmc.fabric.api.client.event.lifecycle.v1.ClientTickEvents
import net.fabricmc.fabric.impl.client.keybinding.KeyBindingRegistryImpl
import net.minecraft.client.MinecraftClient
import net.minecraft.client.option.KeyBinding
import net.minecraft.client.util.InputUtil
import net.moonleay.etaorion.build.BuildConstants
import net.moonleay.etaorion.client.autotext.AutoTextManager
import net.moonleay.etaorion.client.autotext.EOKeyBinding
import net.moonleay.etaorion.client.autotext.KeyBindingManager
import net.moonleay.etaorion.client.modules.component.ModuleManager
import net.moonleay.etaorion.client.modules.component.listener.EndTickListener
import net.moonleay.etaorion.config.AutoTextConfig
import org.apache.logging.log4j.LogManager
import org.lwjgl.glfw.GLFW

internal object ClientMain : ClientModInitializer {
    private val lgr = LogManager.getLogger(BuildConstants.modName)

    override fun onInitializeClient() {
        lgr.info("Starting Eta Orion")
        lgr.info("Registering Modules")
        ModuleManager.registerModules()

        ClientTickEvents.END_CLIENT_TICK.register(EndTickListener())
        lgr.info("Registered EndTickListener #1/2")
        ClientTickEvents.END_CLIENT_TICK.register(ClientTickEvents.EndTick { client: MinecraftClient? ->
            AutoTextManager.onClientTick(client!!)
        })
        lgr.info("Registered EndTickListener #2/2")

        for(i in 0 .. 9){
            val keyBinding: KeyBinding = KeyBinding(
                "etaorion.key_binding_$i",
                InputUtil.Type.KEYSYM,
                GLFW.GLFW_KEY_UNKNOWN,
                "category.etaorion.autotext"
            )
            KeyBindingManager.add(EOKeyBinding(keyBinding, i, false))
            KeyBindingRegistryImpl.registerKeyBinding(keyBinding)
            val autoText = AutoTextConfig.load(i)
            if(autoText == null){
                lgr.info("AutoText with index $i is null.")
                continue
            }
            AutoTextManager.registerNewAutoTxtObject(autoText)
        }

        lgr.info("Registerd all keys")
        lgr.info("EtaOrionFabric has been initialized")
        lgr.info("Eta Orion Ver.${BuildConstants.modVersion}.")
    }
}
