package net.moonleay.etaorion.client.autotext

import org.apache.logging.log4j.LogManager
import org.apache.logging.log4j.Logger

object KeyBindingManager {
    private val lgr: Logger = LogManager.getLogger(KeyBindingManager::class.java)

    private var keyBindings: MutableList<EOKeyBinding> = mutableListOf()

    fun add(keyBinding: EOKeyBinding) {
        if(keyBindings.size >= 10){
            lgr.info("Too many key bindings!")
            return
        }
        keyBindings += keyBinding
    }

    fun bookWithIndex(index: Int): EOKeyBinding?{
        for(keyBinding in keyBindings){
            if(keyBinding.index == index){
                if(keyBinding.isInUse)
                    return null
                keyBinding.isInUse = true
                return keyBinding
            }
        }
        return null
    }

    private fun getFree(): EOKeyBinding? {
        for(keyBinding in keyBindings){
            if(!keyBinding.isInUse){
                lgr.info("Found free keybinding with index ${keyBinding.index}")
                return keyBinding
            }
        }
        return null
    }

    fun bookKeyBinding(): EOKeyBinding?{
        val eoKeyBinding = getFree()?: return null
        eoKeyBinding.isInUse = true
        lgr.info("Booked KeyBinding with index ${eoKeyBinding.index}")
        return eoKeyBinding
    }

    fun cancelKeyBinding(binding: EOKeyBinding){
        lgr.info("Cancelled KeyBinding with index ${binding.index}")
        binding.isInUse = false
        binding.keyBinding.setBoundKey(binding.keyBinding.defaultKey)
    }
}
