package net.moonleay.etaorion.client.modules.component

import net.moonleay.etaorion.client.modules.component.system.AntiFatFinger
import net.moonleay.etaorion.client.modules.feel.NoBob
import net.moonleay.etaorion.client.modules.hud.*
import net.moonleay.etaorion.config.ModuleConfig
import org.apache.logging.log4j.LogManager

object ModuleManager {
    val lgr = LogManager.getLogger(ModuleManager::class.java)
    lateinit var modules: ArrayList<Module>
    var enabledModules = arrayListOf<Module>()
    var enabledGfxModules = mutableMapOf<Position, ArrayList<MovableModule>>()

    fun registerModules() {
        modules = arrayListOf<Module>(
            NoBob(),
            FPS(),
            XYZ(),
            Direction(),
            AntiFatFinger(),
            DurabilityMainHand(),
            DurabilityOffHand(),
        )
        lgr.info("Registered ${modules.size} Modules!")

        for (p in Position.values())
            enabledGfxModules[p] = arrayListOf()

        ModuleConfig.load(modules)
        lgr.info("Loaded config files")
    }

    fun toggleModule(module: Module, enabled: Boolean) {
        if (module is MovableModule) {
            if (enabled) {
                val mapOfPosition = enabledGfxModules[module.position] ?: arrayListOf()
                mapOfPosition.add(module)
                enabledGfxModules[module.position] = mapOfPosition
            } else {
                val mapOfPosition = enabledGfxModules[module.position] ?: return
                mapOfPosition.remove(module)
                enabledGfxModules[module.position] = mapOfPosition
            }
        } else {
            if (enabled) {
                enabledModules.add(module)
            } else {
                enabledModules.remove(module)
            }
        }
    }

    fun updatePositionOfModule(module: MovableModule, oldPosition: Position, targetPosition: Position) {
        var mapOfPosition = enabledGfxModules[oldPosition] ?: return
        mapOfPosition.remove(module)
        enabledGfxModules[oldPosition] = mapOfPosition

        mapOfPosition = enabledGfxModules[targetPosition] ?: arrayListOf()
        mapOfPosition.add(module)
        enabledGfxModules[targetPosition] = mapOfPosition
    }

    fun saveAllModules() {
        ModuleConfig.save(modules)
    }

    fun getPositionFromname(name: String): Position {
        for (p in Position.values()) {
            if (p.Name == name)
                return p
        }
        return Position.OFF
    }

    fun getPositionNumberInt(position: Position): Int {
        var i = 0
        for (p in Position.values()) {
            if (p == position)
                return i
            ++i
        }
        return -1
    }
}
