package net.moonleay.etaorion.client.screen
import com.google.common.collect.ImmutableList
import net.moonleay.etaorion.client.modules.component.*
import net.fabricmc.api.EnvType
import net.fabricmc.api.Environment
import net.minecraft.client.MinecraftClient
import net.minecraft.client.gui.DrawContext
import net.minecraft.client.gui.Element
import net.minecraft.client.gui.Selectable
import net.minecraft.client.gui.screen.Screen
import net.minecraft.client.gui.screen.narration.NarrationMessageBuilder
import net.minecraft.client.gui.screen.narration.NarrationPart
import net.minecraft.client.gui.widget.ButtonWidget
import net.minecraft.client.gui.widget.ElementListWidget
import net.minecraft.client.gui.widget.EntryListWidget
import net.minecraft.text.Text
import net.moonleay.etaorion.client.autotext.AutoText
import net.moonleay.etaorion.client.autotext.AutoTextManager


class AutoTextWidget(parent: Screen, client: MinecraftClient) :
    EntryListWidget<AutoTextWidget.Entry>(client, parent.width + 45, parent.height, 20, parent.height - 32, 20) {

    init {
            for (o in AutoTextManager.autoTextObjects) {
                if(o != null)
                    addEntry(ToggleButtonEntry(o, client))
            }
    }

    override fun getScrollbarPositionX(): Int {
        return super.getScrollbarPositionX() + 15
    }

    override fun getRowWidth(): Int {
        return super.getRowWidth() + 32
    }

    override fun appendNarrations(builder: NarrationMessageBuilder?) {}

    @Environment(EnvType.CLIENT)
    class ToggleButtonEntry internal constructor(autoText: AutoText, client: MinecraftClient) : Entry() {
        private val optionName: Text = Text.of(autoText.textToSend + " (Text ${autoText.eoKeyBinding.index})")
        private val toggleButton: ButtonWidget

        init {
            toggleButton = ButtonWidget.builder(
                Text.of("Remove")
            ) { onButtonClick(autoText, client) }
                .dimensions(0, 0, 75, 20).build()
        }

        private fun onButtonClick(autoText: AutoText, client: MinecraftClient) {
            AutoTextManager.unregisterAutoTextObject(autoText.eoKeyBinding.index)
            client.setScreen(client.currentScreen)
        }

        override fun render(
            context: DrawContext?,
            index: Int,
            y: Int,
            x: Int,
            entryWidth: Int,
            entryHeight: Int,
            mouseX: Int,
            mouseY: Int,
            hovered: Boolean,
            tickDelta: Float
        ) {
            context?.drawTextWithShadow(MinecraftClient.getInstance().textRenderer,
                optionName,
                (x - 60F).toInt(),
                (y + entryHeight / 2 - MinecraftClient.getInstance().textRenderer.fontHeight / 2F).toInt(),
                0xFFFFFF)
            toggleButton.x = x + 150
            toggleButton.y = y
            toggleButton.message = Text.of("Remove")
            toggleButton.render(context, mouseX, mouseY, tickDelta)
        }

        override fun children(): List<Element?> {
            return ImmutableList.of(toggleButton)
        }

        override fun selectableChildren(): List<Selectable?> {
            return ImmutableList.of(toggleButton)
        }

        override fun mouseClicked(mouseX: Double, mouseY: Double, button: Int): Boolean {
            return toggleButton.mouseClicked(mouseX, mouseY, button)
        }
    }

    @Environment(EnvType.CLIENT)
    abstract class Entry : ElementListWidget.Entry<Entry?>()
}
