package net.moonleay.etaorion.client.screen
import com.google.common.collect.ImmutableList
import net.moonleay.etaorion.client.modules.component.*
import net.fabricmc.api.EnvType
import net.fabricmc.api.Environment
import net.minecraft.client.MinecraftClient
import net.minecraft.client.gui.DrawContext
import net.minecraft.client.gui.Element
import net.minecraft.client.gui.Selectable
import net.minecraft.client.gui.screen.Screen
import net.minecraft.client.gui.screen.narration.NarrationMessageBuilder
import net.minecraft.client.gui.screen.narration.NarrationPart
import net.minecraft.client.gui.widget.ButtonWidget
import net.minecraft.client.gui.widget.ElementListWidget
import net.minecraft.client.gui.widget.EntryListWidget
import net.minecraft.text.Text


class ModuleOptionWidget(parent: Screen, client: MinecraftClient) :
    EntryListWidget<ModuleOptionWidget.Entry>(client, parent.width + 45, parent.height, 20, parent.height - 32, 20) {

    init {
        for (type in Category.values()) {
            this.addEntry(CategoryEntry(Text.of(type.Name)))
            for (o in ModuleManager.modules) {
                if (o.category != type)
                    continue
                addEntry(ToggleButtonEntry(o))
            }
        }
    }

    override fun getScrollbarPositionX(): Int {
        return super.getScrollbarPositionX() + 15
    }

    override fun getRowWidth(): Int {
        return super.getRowWidth() + 32
    }

    override fun appendNarrations(builder: NarrationMessageBuilder?) {}

    @Environment(EnvType.CLIENT)
    class ToggleButtonEntry internal constructor(val module: Module) : Entry() {
        private val optionName: Text = Text.of(module.name)
        private val toggleButton: ButtonWidget

        init {
            if (module is MovableModule) {
                val mm: MovableModule = module
                toggleButton = ButtonWidget.builder(
                    Text.of(" " + mm.position.Name)
                ) { onButtonClick(module) }
                    .dimensions(0, 0, 100, 20).build()
            } else {
                toggleButton = ButtonWidget.builder(
                    Text.of(if (module.enabled) " on" else " off")
                ) { onButtonClick(module) }
                    .dimensions(0, 0, 100, 20).build()
            }
        }

        fun onButtonClick(module: Module) {
            if (module is MovableModule) {
                val mm: MovableModule = module
                var intOfCurrentPos = ModuleManager.getPositionNumberInt(mm.position)
                val oldPos = mm.position
                if (intOfCurrentPos + 1 == Position.values().size)
                    intOfCurrentPos = -1
                mm.updatePosition(Position.values()[intOfCurrentPos + 1])
                ModuleManager.updatePositionOfModule(mm, oldPos, Position.values()[intOfCurrentPos + 1])
                return
            }
            module.updateEnabled(!module.enabled)
        }

        override fun render(
            context: DrawContext?,
            index: Int,
            y: Int,
            x: Int,
            entryWidth: Int,
            entryHeight: Int,
            mouseX: Int,
            mouseY: Int,
            hovered: Boolean,
            tickDelta: Float
        ) {
            context?.drawTextWithShadow(MinecraftClient.getInstance().textRenderer,
                optionName,
                (x - 60F).toInt(),
                (y + entryHeight / 2 - MinecraftClient.getInstance().textRenderer.fontHeight / 2F).toInt(),
                0xFFFFFF)
            toggleButton.x = x + 150
            toggleButton.y = y
            if (module is MovableModule) {
                val mm: MovableModule = module
                toggleButton.message = Text.of(mm.position.Name)
            } else {
                toggleButton.message = Text.of(if (module.enabled) "on" else "off")
            }
            toggleButton.render(context, mouseX, mouseY, tickDelta)
        }

        override fun children(): List<Element?> {
            return ImmutableList.of(toggleButton)
        }

        override fun selectableChildren(): List<Selectable?> {
            return ImmutableList.of(toggleButton)
        }

        override fun mouseClicked(mouseX: Double, mouseY: Double, button: Int): Boolean {
            return toggleButton.mouseClicked(mouseX, mouseY, button)
        }
    }

    @Environment(EnvType.CLIENT)
    abstract class Entry : ElementListWidget.Entry<Entry?>()


    @Environment(EnvType.CLIENT)
    class CategoryEntry(text: Text) : Entry() {
        private val text: Text
        private val textWidth: Int

        init {
            this.text = text
            textWidth = MinecraftClient.getInstance().textRenderer.getWidth(this.text)
        }

        override fun render(
            context: DrawContext?,
            index: Int,
            y: Int,
            x: Int,
            entryWidth: Int,
            entryHeight: Int,
            mouseX: Int,
            mouseY: Int,
            hovered: Boolean,
            tickDelta: Float
        ) {
            context?.drawTextWithShadow(MinecraftClient.getInstance().textRenderer, text,
                (MinecraftClient.getInstance().currentScreen!!.width / 2 - textWidth / 2F).toInt(),
                (y + entryHeight - MinecraftClient.getInstance().textRenderer.fontHeight - 1F).toInt(),
                0xFFFFFF)
        }

        override fun children(): List<Element?> {
            return emptyList()
        }

        override fun selectableChildren(): List<Selectable?> {
            return ImmutableList.of(object : Selectable {
                override fun getType(): Selectable.SelectionType {
                    return Selectable.SelectionType.HOVERED
                }

                override fun appendNarrations(builder: NarrationMessageBuilder) {
                    builder.put(NarrationPart.TITLE, text)
                }
            })
        }
    }

}
