package net.moonleay.etaorion.config

import net.moonleay.etaorion.client.modules.component.ModuleManager
import net.moonleay.etaorion.client.modules.component.MovableModule
import net.moonleay.etaorion.client.modules.component.Module
import java.io.*
import java.util.*

object ModuleConfig {
    private val lgr = org.apache.logging.log4j.LogManager.getLogger(ModuleConfig::class.java)
    private const val basePath = "./config/EtaOrionFabric/"

    fun load(modules: List<Module>) {
        val baseDir = File(basePath)
        if (!baseDir.exists()) {
            save(modules)
            return
        }
        var moduleFile: File
        var input: InputStream?
        var prop: Properties
        for (m in modules) {
            moduleFile = File(baseDir, m.name + ".EOF")
            if (!moduleFile.exists()) {
                save(modules)
                return
            }
            lgr.info("Loading " + m.name + ", File: " + moduleFile)
            try {
                input = FileInputStream(moduleFile)
                prop = Properties()
                prop.load(input)
                if (m is MovableModule) {
                    val mm: MovableModule = m
                    mm.onLoadGfx(
                        prop.getProperty("enabled").equals("true"),
                        ModuleManager.getPositionFromname(prop.getProperty("position"))
                    )
                } else {
                    m.onLoad(
                        prop.getProperty("enabled").equals("true")
                    )
                }
            } catch (e: IOException) {
                throw RuntimeException(e)
            }
        }
        lgr.info("Loaded Module Settings")
    }

    fun save(modules: List<Module>) {
        val baseDir = File(basePath)
        if (!baseDir.exists()) baseDir.mkdirs()
        var moduleFile: File
        var output: OutputStream?
        var prop: Properties
        for (m in modules) {
            moduleFile = File(baseDir, m.name + ".EOF")
            if (!moduleFile.exists()) {
                try {
                    moduleFile.createNewFile()
                } catch (e: IOException) {
                    throw RuntimeException(e)
                }
            }
            lgr.info("Saving " + m.name + ", File: " + moduleFile)
            try {
                output = FileOutputStream(moduleFile)
                prop = Properties()
                prop.setProperty("enabled", if (m.enabled) "true" else "false")
                if (m is MovableModule) {
                    prop.setProperty("position", m.position.Name)
                }
                prop.store(output, null)
            } catch (e: IOException) {
                throw RuntimeException(e)
            }
        }
        lgr.info("Saved Module Settings")
    }
}
