package net.moonleay.etaorion.mixin;

import net.minecraft.client.MinecraftClient;
import net.minecraft.client.font.TextRenderer;
import net.minecraft.client.gui.DrawContext;
import net.minecraft.client.gui.hud.InGameHud;
import net.minecraft.text.Text;
import net.moonleay.etaorion.client.modules.component.ModuleManager;
import net.moonleay.etaorion.client.modules.component.MovableModule;
import net.moonleay.etaorion.client.modules.component.Position;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.Shadow;
import org.spongepowered.asm.mixin.Unique;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;

@Mixin(InGameHud.class)
public abstract class ModuleRenderer {
    private Logger lgr = LogManager.getLogger(ModuleRenderer.class);

    @Shadow
    public abstract TextRenderer getTextRenderer();

    @Inject(at = @At("HEAD"), method = "renderStatusEffectOverlay")
    private void renderModules(DrawContext context, CallbackInfo ci) {
        MinecraftClient mc = MinecraftClient.getInstance();
        TextRenderer tr = mc.textRenderer;
        int scale = (mc.options.getGuiScale().getValue() == 0 ? 1 : mc.options.getGuiScale().getValue());
        int screenWidth = mc.getWindow().getWidth();
        int screenHeight = mc.getWindow().getHeight();
        int baseYtop = 4;
        int baseYcenter = screenHeight / 2 / scale;
        int offset = 12;
        int index = 0;
        int xCentered = screenWidth / 2 / scale;
        int bottomY = screenHeight / scale - 4 - tr.fontHeight - (offset * index);
        for (MovableModule m : ModuleManager.INSTANCE.getEnabledGfxModules().get(Position.TOPLEFT)) {
            context.drawTextWithShadow(tr, Text.of(m.getText4Render()), 4, baseYtop + (offset * index), -1);
            ++index;
        }

        index = 0;
        for (MovableModule m : ModuleManager.INSTANCE.getEnabledGfxModules().get(Position.TOPCENTER)) {
            context.drawCenteredTextWithShadow(tr, Text.of(m.getText4Render()), xCentered, baseYtop + (offset * index), -1);
            ++index;
        }

        index = 0;
        for (MovableModule m : ModuleManager.INSTANCE.getEnabledGfxModules().get(Position.TOPRIGHT)) {
            context.drawTextWithShadow(tr, Text.of(m.getText4Render()), getXRight(m.getText4Render(), screenWidth), baseYtop + (offset * index), -1);
            ++index;
        }

        index = 0;
        int yBaseCentered = baseYcenter - ((ModuleManager.INSTANCE.getEnabledGfxModules().get(Position.RIGHTCENTER).size() / 2) * 12);
        for (MovableModule m : ModuleManager.INSTANCE.getEnabledGfxModules().get(Position.RIGHTCENTER)) {
            context.drawTextWithShadow(tr, Text.of(m.getText4Render()), getXRight(m.getText4Render(), screenWidth), yBaseCentered + (offset * index), -1);
            ++index;
        }

        index = 0;
        for (MovableModule m : ModuleManager.INSTANCE.getEnabledGfxModules().get(Position.BOTTOMRIGHT)) {
            context.drawTextWithShadow(tr, Text.of(m.getText4Render()), getXRight(m.getText4Render(), screenWidth), bottomY - (offset * index), -1);
            ++index;
        }

        index = 0;
        for (MovableModule m : ModuleManager.INSTANCE.getEnabledGfxModules().get(Position.BOTTOMCENTER)) {
            context.drawCenteredTextWithShadow(tr, Text.of(m.getText4Render()), xCentered, screenHeight / scale - 4 - 60 - tr.fontHeight - (offset * index), -1);
            ++index;
        }

        index = 0;
        for (MovableModule m : ModuleManager.INSTANCE.getEnabledGfxModules().get(Position.BOTTOMLEFT)) {
            context.drawTextWithShadow(tr, Text.of(m.getText4Render()), 4, bottomY - (offset * index), -1);
            ++index;
        }

        index = 0;
        yBaseCentered = baseYcenter - ((ModuleManager.INSTANCE.getEnabledGfxModules().get(Position.LEFTCENTER).size() / 2) * 12);
        for (MovableModule m : ModuleManager.INSTANCE.getEnabledGfxModules().get(Position.LEFTCENTER)) {
            context.drawTextWithShadow(tr, Text.of(m.getText4Render()), 4, yBaseCentered + (offset * index), -1);
            ++index;
        }
    }

    @Unique
    private int getXRight(String txt, int screenWidth) {
        return screenWidth / (MinecraftClient.getInstance().options.getGuiScale().getValue() == 0 ? 1 : MinecraftClient.getInstance().options.getGuiScale().getValue()) - 4 - MinecraftClient.getInstance().textRenderer.getWidth(Text.of(txt));
    }
}
